package net.scp;


import futils.Futil;
import gui.In;
import gui.dialogs.LabelledItemPanel;
import gui.dialogs.ModalJDialog;
import gui.run.RunButton;
import gui.run.RunPasswordField;
import gui.run.RunTextField;

import javax.swing.*;
import java.beans.PropertyChangeSupport;
import java.io.File;

/**
 * This class demonstrates the usage of the StandardDialog class.
 */
public class ScpDialog extends ModalJDialog {

    PropertyChangeSupport pcs = new PropertyChangeSupport(this);
    ScpModel scpModel ;

 

    private LabelledItemPanel c = new LabelledItemPanel();

    public ScpDialog() {
        init();
    }

    private void init() {
        setTitle("Scp Dialog");
        scpModel = ScpModel.restore();
        c.addItem("host", new RunTextField(scpModel.getHost(),20) {
            public void run() {
                scpModel.setHost(getText());
            }
        });
        final RunTextField localFileTextField = getLocalFileTextField();
        c.addItem("localFile", localFileTextField);
        c.addItem("setLocalFile", new RunButton("browse") {
            public void run() {
                File f = Futil.getReadFile("select local file");
                localFileTextField.setText(f.getAbsolutePath());
                scpModel.setLocalFile(f.getAbsolutePath());

            }
        });
        c.addItem("remoteFile", new RunTextField(scpModel.getRemoteFile(),20) {
            public void run() {
                scpModel.setRemoteFile(getText());
            }
        });
        c.addItem("user", new RunTextField(scpModel.getUser(),20) {
            public void run() {
                scpModel.setUser(getText());
            }
        });
        c.addItem("password", new RunPasswordField(scpModel.getPassword(),20) {
            public void run() {
                scpModel.setPassword(getPasswordString());
            }
        });
        c.setBorder(BorderFactory.createEtchedBorder());

        setContentPane(c);
    }

    private RunTextField getLocalFileTextField() {
        return new RunTextField(
                scpModel.getLocalFile(),20) {
            public void run() {
                scpModel.setLocalFile(getText());
            }
        };
    }

    /**
     * This method gets the values of the panel entry fields.
     *
     * @return an object containing the Customer data
     */
    public ScpModel getData() {
        return scpModel;
    }
    public void addScpChangeEventListener(ScpModelChangeEventListener l){
        pcs.addPropertyChangeListener(l);
    }
    public void okPressed(){
        if (isValidData()){
            processOk();
        }
        else
            debugData();
    }

    private void processOk() {
        System.out.println("OK process!");
        scpModel.save();
        final ScpModelChangeEvent evt = new ScpModelChangeEvent(this,scpModel);
        pcs.firePropertyChange(evt);
        System.out.println("uploading file");
        scpModel.upload();
        System.out.println("upload complete");
        //super.okPressed();

    }

    public void debugData() {
        In.message("data not valid! Model="+scpModel);

    }


    /**
     * This method checks that the data entered is valid.
     *
     * @return <code>true</code> if the data is valid, otherwise
     *         <code>false</code>
     */
    protected boolean isValidData() {

        return scpModel.isValidData();
    }

    public static void main(String[] args) {
        ScpDialog dialog = new ScpDialog();
        dialog.pack();
        dialog.show();

        System.exit(0);
    }
}