package video;

import futils.Futil;
import gui.In;
import gui.dialogs.LabelledItemPanel;
import gui.dialogs.ModalJDialog;
import gui.run.RunButton;
import gui.run.RunCheckBox;
import gui.run.RunSlider;
import gui.run.RunTextField;

import javax.swing.BorderFactory;
import java.io.File;

/**
 * This class demonstrates the usage of the StandardDialog class.
 */
public class NequantDialog extends ModalJDialog {
    NeuquantCaptureBean ncb;


    public NequantDialog() {
        init();
    }

    private void init() {
        setTitle("NeQuantDialog");
        ncb = NeuquantCaptureBean.restore();
        final RunTextField outputFileTextField = getOutputFileTextField();
        LabelledItemPanel c = new LabelledItemPanel();
        c.addItem("localFile", outputFileTextField);
        c.addItem("setLocalFile", new RunButton("browse") {
            public void run() {
                File f = Futil.getReadFile("select local file");
                outputFileTextField.setText(f.getAbsolutePath());
                ncb.setOutputFile(f.getAbsolutePath());
            }
        });
        c.addItem("is Looped", new RunCheckBox("", ncb.isLooped()) {
            public void run() {
                ncb.setLooped(isSelected());
            }
        });
        c.addItem("sec/frame", new RunSlider(0, 10, (int) ncb.getSecondsPerFrame()) {
            public void run() {
                ncb.setSecondsPerFrame(getValue());
            }
        });
        c.setBorder(BorderFactory.createEtchedBorder());
        setContentPane(c);
    }

    private RunTextField getOutputFileTextField() {
        return new RunTextField(ncb.getOutputFile(), 20) {
            public void run() {
                ncb.setOutputFile(getText());
            }
        };
    }

    /**
     * This method gets the values of the panel entry fields.
     *
     * @return an object containing the Customer data
     */
    public NeuquantCaptureBean getData() {
        return ncb;
    }

    public void okPressed() {
        if (isValidData()) {
            processOk();
        } else
            debugData();
    }

    private void processOk() {
        System.out.println("OK process!");
        ncb.save();
        System.out.println("Validate the data and capture...");
        //super.okPressed();
    }

    public void debugData() {
        In.message("data not valid! Model=" + ncb);
    }

    /**
     * This method checks that the data entered is valid.
     *
     * @return <code>true</code> if the data is valid, otherwise
     *         <code>false</code>
     */
    protected boolean isValidData() {
        return ncb.isValidData();
    }

    public static void main(String[] args) {
        NequantDialog dialog = new NequantDialog();
        dialog.pack();
        dialog.show();
        System.exit(0);
    }
}